/* Copyright (c) 2000-2006 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

//---- globals
var _findingInterps = false;
var prefExecutable = null;
var programmingLanguage = "Perl";
var _bundle = Components.classes["@mozilla.org/intl/stringbundle;1"]
            .getService(Components.interfaces.nsIStringBundleService)
            .createBundle("chrome://komodo/locale/pref/pref-languages.properties");
//---- functions

function OnPreferencePageOK(prefset)
{
    // ensure that the default perl interpreter is valid
    return checkValidInterpreterSetting(prefset,
                                        "perlDefaultInterpreter",
                                        programmingLanguage);
}

// Populate the (tree) list of available Perl interpreters on the current
// system.
function PrefPerl_PopulatePerlInterps()
{
    var availInterpList = document.getElementById("perlDefaultInterpreter");

    // remove any existing items and add a "finding..." one
    _findingInterps = true;
    availInterpList.removeAllItems();
    availInterpList.appendItem(_bundle.formatStringFromName("findingInterpreters.label", [programmingLanguage], 1));

    // get a list of installed Perl interpreters
    var sysUtils = Components.classes['@activestate.com/koSysUtils;1'].
        getService(Components.interfaces.koISysUtils);
    var availInterps = new Array();
    availInterps = sysUtils.WhichAll("perl", new Object());

    availInterpList.removeAllItems();
    availInterpList.appendItem(_bundle.GetStringFromName("findOnPath.label"),'');

    var found = false;
    // populate the tree listing them
    if (availInterps.length == 0) {
        // tell the user no interpreter was found and direct them to
        // ActiveState to get one
        document.getElementById("no-avail-interps-message").removeAttribute("collapsed");
    } else {
        for (var i = 0; i < availInterps.length; i++) {
            availInterpList.appendItem(availInterps[i],availInterps[i]);
            if (availInterps[i] == prefExecutable) found = true;
        }
    }
    if (!found && prefExecutable)
        availInterpList.appendItem(prefExecutable,prefExecutable);
    _findingInterps = false;
}


function PrefPerl_OnLoad()
{
    if (parent.hPrefWindow.prefset.hasStringPref('perlDefaultInterpreter') &&
        parent.hPrefWindow.prefset.getStringPref('perlDefaultInterpreter'))
        prefExecutable = parent.hPrefWindow.prefset.getStringPref('perlDefaultInterpreter');
    else
        prefExecutable = '';
    PrefPerl_PopulatePerlInterps();
    var appInfoEx = Components.classes["@activestate.com/koAppInfoEx?app=Perl;1"].
            getService(Components.interfaces.koIPerlInfoEx);
    _setPerlCriticSection(appInfoEx.isPerlCriticInstalled(/*forceCheck=*/true));

    var origWindow = ko.windowManager.getMainWindow();
    var cwd = origWindow.ko.window.getCwd();
    parent.hPrefWindow.onpageload();
    var extraPaths = document.getElementById("perlExtraPaths");
    extraPaths.init(); // must happen after onpageload
    extraPaths.setCwd(cwd);
}

function loadPerlExecutable()
{
    loadExecutableIntoInterpreterList("perlDefaultInterpreter");
}

function onPerlDefaultInterpreterChanged() {
    var availInterpList = document.getElementById("perlDefaultInterpreter");
    var newInterpreter = availInterpList.selectedItem.value;
    // We can't use koAppInfo service, because it's still pointing at
    // the old Perl interpreter value.
    var cmd = !newInterpreter ? "perl" : newInterpreter;
    cmd += " -Mcriticism -e 1";
    var runSvc = Components.classes["@activestate.com/koRunService;1"]
               .getService(Components.interfaces.koIRunService);
    var out = {}, err = {};
    var res = runSvc.RunAndCaptureOutput(cmd, null, null, null, out, err);
    _setPerlCriticSection(res == 0);
}

function _setPerlCriticSection(havePerlCritic) {
    var perlCriticLabel = document.getElementById("perl_lintOptions_perlCriticBox_label");
    var perlCriticMenu = document.getElementById("perl_lintOption_perlCriticLevel");
    var perlCriticEnableNode = document.getElementById("perl_lintOption_perlCriticEnableNote");
    perlCriticLabel.disabled = !havePerlCritic;
    perlCriticMenu.disabled = !havePerlCritic;
    if (havePerlCritic) {
        perlCriticEnableNode.setAttribute('collapsed', true);
    } else {
        perlCriticEnableNode.removeAttribute('collapsed');
    }
}

function loadPerlLogpath()
{
    var perlLog = ko.filepicker.getFolder();
    if (perlLog != null) {
        var textbox = document.getElementById("perl_debuggerlogpath");
        textbox.value = perlLog;
    }
}
